@@ -11,10 +11,8 @@
 
 #include <QMetaEnum>
 #include <QProcess>
+#include <QSignalSpy>
 #include <QTest>
-#include <QtPlugin>
-
-Q_IMPORT_PLUGIN(OSM_XmlIOPlugin)
 
 using namespace KOSMIndoorMap;
 
@@ -93,7 +91,9 @@ private Q_SLOTS:
         QFETCH(QString, expected);
 
         MapLoader loader;
+        QSignalSpy doneSpy(&loader, &MapLoader::done);
         loader.loadFromFile(input);
+        QVERIFY(doneSpy.wait());
         QCOMPARE(loader.isLoading(), false);
         QCOMPARE(loader.hasError(), false);
 
