@@ -10,13 +10,18 @@
 #include "localization.h"
 #include "osmaddress.h"
 
+#include <wikidata/wikidataquery.h>
+
 #include <KLocalizedString>
 
+#include <QUrlQuery>
+
 #include <cctype>
 
+using namespace Qt::Literals::StringLiterals;
 using namespace KOSMIndoorMap;
 
-static QString formatDistance(int meter)
+[[nodiscard]] static QString formatDistance(int meter)
 {
     if (meter < 1000) {
         return i18n("%1m", meter);
@@ -45,6 +50,7 @@ OSMElementInformationModel::OSMElementInformationModel(QObject *parent)
     : QAbstractListModel(parent)
     , m_langs(OSM::Languages::fromQLocale(QLocale()))
 {
+    m_wikidataMgr.setUserAgentEmailAddress(u"kde-pim@kde.org"_s);
 }
 
 OSMElementInformationModel::~OSMElementInformationModel() = default;
@@ -97,7 +103,7 @@ int OSMElementInformationModel::rowCount(const QModelIndex &parent) const
     if (parent.isValid() || m_element.type() == OSM::Type::Null) {
         return 0;
     }
-    return m_infos.size();
+    return (int)m_infos.size();
 }
 
 QVariant OSMElementInformationModel::data(const QModelIndex &index, int role) const
@@ -121,6 +127,11 @@ QVariant OSMElementInformationModel::data(const QModelIndex &index, int role) co
                     return PostalAddress;
                 case OpeningHours:
                     return OpeningHoursType;
+                case Image:
+                case Logo:
+                    return ImageType;
+                case DebugKey:
+                    return debugTagUrl(index.row()).isValid() ? Link : String;
                 default:
                     return String;
             }
@@ -138,6 +149,9 @@ QVariant OSMElementInformationModel::data(const QModelIndex &index, int role) co
                 default: return valueForKey(info);
             }
         case ValueUrlRole:
+            if (info.key == DebugKey) {
+                return debugTagUrl(index.row());
+            }
             return urlify(valueForKey(info), info.key);
         case CategoryRole:
             return info.category;
@@ -167,8 +181,8 @@ struct KeyCategoryMapEntry {
     OSMElementInformationModel::Key m_key;
     OSMElementInformationModel::KeyCategory m_category;
 
-    constexpr inline OSMElementInformationModel::Key key() const { return m_key; }
-    constexpr inline OSMElementInformationModel::KeyCategory category() const { return m_category; }
+    [[nodiscard]] constexpr inline OSMElementInformationModel::Key key() const { return m_key; }
+    [[nodiscard]] constexpr inline OSMElementInformationModel::KeyCategory category() const { return m_category; }
 };
 
 static constexpr const KeyCategoryMapEntry simple_key_map[] = {
@@ -177,6 +191,7 @@ static constexpr const KeyCategoryMapEntry simple_key_map[] = {
     M("amenity", Category, Header),
     M("bicycle_parking", BicycleParking, Parking),
     M("brand", Name, Header),
+    M("brand:wikidata", Image, Main),
     M("brand:wikipedia", Wikipedia, UnresolvedCategory),
     M("building", Category, Header),
     M("bus_lines", Routes, Main),
@@ -202,14 +217,18 @@ static constexpr const KeyCategoryMapEntry simple_key_map[] = {
     M("email", Email, Contact),
     M("fee", Fee, UnresolvedCategory),
     M("genus", Name, Header),
+    M("genus:wikidata", Image, Main),
     M("historic", Category, Header),
+    M("image", Image, Main),
     M("int_name", Name, Header),
     M("leisure", Category, Header),
     M("maxstay", MaxStay, Parking),
+    M("memorial:text", Description, Main),
     M("mx:realtime_available", AvailableVehicles, Main),
     M("mx:remaining_range", RemainingRange, Main),
     M("mx:vehicle", Category, Header),
     M("network", Network, Operator),
+    M("network:wikidata", Image, Main),
     M("network:wikipedia", OperatorWikipedia, Operator),
     M("office", Category, Header),
     M("old_name", OldName, UnresolvedCategory),
@@ -218,6 +237,7 @@ static constexpr const KeyCategoryMapEntry simple_key_map[] = {
     M("operator:email", Email, Contact),
     M("operator:phone", Phone, Contact),
     M("operator:website", Website, Contact),
+    M("operator:wikidata", Image, Main),
     M("operator:wikipedia", OperatorWikipedia, Operator),
     M("parking:fee", Fee, Parking),
     M("payment:cash", PaymentCash, Payment),
@@ -227,15 +247,19 @@ static constexpr const KeyCategoryMapEntry simple_key_map[] = {
     M("room", Category, Header),
     M("route_ref", Routes, Main),
     M("shop", Category, Header),
+    M("species:wikidata", Image, Main),
     M("tactile_writing", TactileWriting, Accessibility), // occurs also unqualified
     M("takeaway", Takeaway, Main),
     M("toilets:fee", Fee, Toilets),
     M("toilets:wheelchair", Wheelchair, Toilets),
     M("tourism", Category, Header),
     M("url", Website, Contact),
+    M("vending", VendingMachineOffer, Header),
     M("website", Website, Contact),
     M("wheelchair", Wheelchair, Accessibility),
     M("wheelchair:lift", WheelchairLift, Accessibility),
+    M("wikidata", Image, Main),
+    M("wikimedia_commons", Image, Main),
 };
 static_assert(isSortedLookupTable(simple_key_map), "key map is not sorted!");
 
@@ -290,16 +314,19 @@ void OSMElementInformationModel::reload()
         addEntryForKey((*it).key.name(), diet_type_map);
         addEntryForKey((*it).key.name(), socket_type_map);
         addEntryForKey((*it).key.name(), authentication_type_map);
-        addEntryForKey((*it).key.name(), gender_type_map);
         addEntryForLocalizedKey((*it).key.name(), tactile_writing_map);
 
         if (isRoom && std::strcmp((*it).key.name(), "ref") == 0) {
             m_infos.push_back(Info{Name, Header});
         }
     }
+    if (Localization::hasGenderSegregrationKey(m_element)) {
+         m_infos.emplace_back(Gender, UnresolvedCategory);
+    }
 
     std::sort(m_infos.begin(), m_infos.end());
     m_infos.erase(std::unique(m_infos.begin(), m_infos.end()), m_infos.end());
+    resolveOnlineContent();
     resolveCategories();
     resolveHeaders();
 
@@ -326,6 +353,75 @@ void OSMElementInformationModel::reload()
     }
 }
 
+void OSMElementInformationModel::resolveOnlineContent()
+{
+    if (!m_allowOnlineContent) {
+        m_infos.erase(std::remove_if(m_infos.begin(), m_infos.end(), [](const auto &info) {
+            return info.key == Image || info.key == Logo;
+        }), m_infos.end());
+        return;
+    }
+
+    const auto commons = m_element.tagValue("wikimedia_commons");
+    const auto hasValidCommons = commons.startsWith("File:");
+    const auto image = m_element.tagValue("image");
+    const auto hasValidImage = image.contains("://commons.wikimedia.org/");
+    const auto wdId = m_element.tagValue("wikidata", "species:wikidata", "genus:wikidata", "subject:wikidata", "operator:wikidata", "network:wikidata", "brand:wikidata");
+
+    // query Wikidata content
+    if (!hasValidCommons && !hasValidImage && !wdId.isEmpty()) {
+        auto query = new Wikidata::EntitiesQuery(this);
+        query->setItems({Wikidata::Q{wdId}});
+        connect(query, &Wikidata::EntitiesQuery::finished, this, [query, this]() {
+            query->deleteLater();
+            auto res = query->takeResult();
+            for (const auto &item : res) {
+                std::vector<Wikidata::P> props({Wikidata::P::image, Wikidata::P::imageOfInterior, Wikidata::P::aerialView, Wikidata::P::view, Wikidata::P::modelImage});
+                // prefer night time view images at night
+                if (const auto now = QTime::currentTime(); now.hour() >= 22 || now.hour() <= 6) {
+                    props.insert(props.begin(), Wikidata::P::nighttimeView);
+                } else {
+                    props.emplace_back(Wikidata::P::nighttimeView);
+                }
+                // same for winter view
+                if (const auto today = QDate::currentDate(); today.month() > 10 || today.month() < 3) {
+                    props.insert(props.begin(), Wikidata::P::winterView);
+                } else {
+                    props.emplace_back(Wikidata::P::nighttimeView);
+                }
+                // prefer the logo for brand/operator/network
+                bool isLogo = false;
+                if (item.id() == Wikidata::Q(m_element.tagValue("operator:wikidata", "network:wikidata", "brand:wikidata"))) {
+                    props = {Wikidata::P::logoImage};
+                    isLogo = true;
+                } else {
+                    props.emplace_back(Wikidata::P::logoImage);
+                }
+
+                for (const auto p : props) {
+                    const auto img = item.value<QString>(p);
+                    if (img.isEmpty()) {
+                        continue;
+                    }
+                    m_wikidataImageMap.insert(item.id(), img);
+                    const auto it = std::find_if(m_infos.begin(), m_infos.end(), [](const auto &info) { return info.key == Image; });
+                    if (isLogo) {
+                        (*it).key = Logo;
+                    }
+                    const auto idx = index((int)std::distance(m_infos.begin(), it), 0);
+                    Q_EMIT dataChanged(idx, idx);
+                    break;
+                }
+            }
+        });
+        m_wikidataMgr.execute(query);
+    }
+
+    if (!hasValidCommons && !hasValidImage && wdId.isEmpty()) {
+        m_infos.erase(std::remove_if(m_infos.begin(), m_infos.end(), [](const auto &info) { return info.key == Image; }), m_infos.end());
+    }
+}
+
 void OSMElementInformationModel::resolveCategories()
 {
     if (m_infos.empty() || m_infos[0].category != UnresolvedCategory) {
@@ -374,7 +470,8 @@ void OSMElementInformationModel::resolveCategories()
 
 void OSMElementInformationModel::resolveHeaders()
 {
-    for (auto key : { Name, Network, OperatorName, Category }) {
+    // find the best name candidate
+    for (auto key : { Name, VendingMachineOffer, Network, OperatorName, Category }) {
         if (m_nameKey != NoKey) {
             break;
         }
@@ -391,16 +488,21 @@ void OSMElementInformationModel::resolveHeaders()
         break;
     }
 
-    // we use the categories as header if there is no name, so don't duplicate that
-    const auto it = std::find_if(m_infos.begin(), m_infos.end(), [](Info info) {
-        return info.key == Category;
-    });
-    if (it == m_infos.end() || m_nameKey == Category) {
-        return;
-    }
+    // same again for the category header
+    for (auto key : { VendingMachineOffer, Category }) {
+        const auto it = std::find_if(m_infos.begin(), m_infos.end(), [key](Info info) {
+            return info.key == key;
+        });
+        if (it == m_infos.end()) {
+            continue;
+        }
 
-    m_infos.erase(it);
-    m_categoryKey = Category;
+        if (m_categoryKey == NoKey && m_nameKey != key) {
+            m_categoryKey = (*it).key;
+        }
+
+        m_infos.erase(it);
+    }
 }
 
 bool OSMElementInformationModel::promoteMainCategory(OSMElementInformationModel::KeyCategory cat)
@@ -459,12 +561,34 @@ QString OSMElementInformationModel::debugTagValue(int row) const
     return QString::fromUtf8((*(m_element.tagsBegin() + tagIdx)).value);
 }
 
+QUrl OSMElementInformationModel::debugTagUrl(int row) const
+{
+    const auto tagCount = std::distance(m_element.tagsBegin(), m_element.tagsEnd());
+    const auto tagIdx = row - (rowCount() - tagCount);
+    const auto key = QByteArrayView((*(m_element.tagsBegin() + tagIdx)).key.name());
+    const auto value = (*(m_element.tagsBegin() + tagIdx)).value;
+    if (key.endsWith(":wikipedia") || key == "wikipedia") {
+        return wikipediaUrl(value);
+    }
+    if (key.endsWith(":wikidata") || key == "wikidata") {
+        return QUrl(u"https://wikidata.org/wiki/" + QString::fromUtf8(value));
+    }
+    if (value.startsWith("http"_L1)) {
+        return QUrl(QString::fromUtf8(value));
+    }
+    return {};
+}
+
 QString OSMElementInformationModel::keyName(OSMElementInformationModel::Key key) const
 {
     switch (key) {
         case NoKey:
         case Name:
-        case Category: return {};
+        case Category:
+        case VendingMachineOffer:
+        case Image:
+        case Logo:
+            return {};
         case OldName: return i18n("Formerly");
         case Description: return i18n("Description");
         case Routes: return i18n("Routes");
@@ -523,7 +647,7 @@ static void appendNonEmpty(const QByteArray &tagValue, QList<QByteArray> &l)
     }
 }
 
-static QChar::Script scriptForString(QStringView s)
+[[nodiscard]] static QChar::Script scriptForString(QStringView s)
 {
     return std::accumulate(s.begin(), s.end(), QChar::Script_Unknown, [](QChar::Script script, QChar c) { return std::max(script, c.script());});
 }
@@ -538,11 +662,29 @@ struct {
     { QLocale::CyrillicScript, QChar::Script_Cyrillic },
 };
 
-static bool isSameScript(QLocale::Script ls, QChar::Script cs)
+[[nodiscard]] static bool isSameScript(QLocale::Script ls, QChar::Script cs)
 {
     return std::find_if(std::begin(script_map), std::end(script_map), [ls, cs](const auto &m) { return m.localeScript == ls && m.charScript == cs; }) != std::end(script_map);
 }
 
+[[nodiscard]] static QUrl wikimediaCommondRedirect(const QString &fileName)
+{
+    if (fileName.isEmpty()) {
+        return {};
+    }
+
+    QUrl redirectUrl;
+    redirectUrl.setScheme(u"https"_s);
+    redirectUrl.setHost(u"commons.wikimedia.org"_s);
+    redirectUrl.setPath(u"/wiki/Special:Redirect/file"_s);
+    QUrlQuery query;
+    query.addQueryItem(u"wptype"_s, u"file"_s);
+    query.addQueryItem(u"wpvalue"_s, fileName);
+    query.addQueryItem(u"width"_s, u"512"_s);
+    redirectUrl.setQuery(query);
+    return redirectUrl;
+}
+
 QVariant OSMElementInformationModel::valueForKey(Info info) const
 {
     switch (info.key) {
@@ -565,7 +707,7 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             appendNonEmpty(m_element.tagValue("amenity"), l);
             appendNonEmpty(m_element.tagValue("shop"), l);
             appendNonEmpty(m_element.tagValue("tourism"), l);
-            appendNonEmpty(m_element.tagValue("vending"), l);
+            // appendNonEmpty(m_element.tagValue("vending"), l);
             const auto diplomatic = m_element.tagValue("diplomatic");
             appendNonEmpty(diplomatic, l);
             if (diplomatic.isEmpty()) {
@@ -584,7 +726,7 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             // TODO drop general categories if specific ones are available (e.g. restaurant vs fast_food)
 
             for (auto it = l.begin(); it != l.end();++it) {
-                if ((*it).isEmpty() || (*it) == "yes" || (*it) == "no" || (*it) == "vending_machine" || (*it) == "building") {
+                if ((*it).isEmpty() || (*it) == "yes" || (*it) == "no" || (*it) == "building") {
                     continue;
                 }
                 out.push_back(Localization::amenityType((*it).constData()));
@@ -604,13 +746,27 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             out.erase(std::unique(out.begin(), out.end()), out.end());
             return QLocale().createSeparatedList(out);
         }
+        case Image:
+        case Logo:
+        {
+            const auto commons = m_element.tagValue("wikimedia_commons");
+            if (commons.startsWith("File:")) {
+                return wikimediaCommondRedirect(QString::fromUtf8(QByteArrayView(commons).mid(5)));
+            }
+            const QUrl url(QString::fromUtf8(m_element.tagValue("image")));
+            if (url.host() == "commons.wikimedia.org"_L1) {
+                return wikimediaCommondRedirect(url.fileName());
+            }
+            const auto wdId = m_element.tagValue("wikidata", "species:wikidata", "genus:wikidata", "subject:wikidata", "operator:wikidata", "network:wikidata", "brand:wikidata");
+            return wikimediaCommondRedirect(m_wikidataImageMap.value(Wikidata::Q{wdId}));
+        }
         case OldName:
         {
             const auto l = QString::fromUtf8(m_element.tagValue("old_name")).split(QLatin1Char(';'));
             return l.join(QLatin1String(", "));
         }
         case Description:
-            return m_element.tagValue(m_langs, "description");
+            return m_element.tagValue(m_langs, "description", "memorial:text");
         case Routes:
         {
             auto l = QString::fromUtf8(m_element.tagValue("route_ref", "bus_routes", "bus_lines", "buses")).split(QLatin1Char(';'), Qt::SkipEmptyParts);
@@ -620,6 +776,8 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             return QLocale().createSeparatedList(l);
         }
         case Cuisine: return Localization::cuisineTypes(m_element.tagValue("cuisine"));
+        case VendingMachineOffer:
+            return Localization::amenityTypes(m_element.tagValue("vending"), Localization::ReturnEmptyOnUnknownKey);
         case Diet:
         {
             QStringList l;
@@ -739,17 +897,7 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             // TODO look for changing_table:location too
             return translatedBoolValue(m_element.tagValue("changing_table", "diaper"));
         case Gender:
-        {
-            QStringList l;
-            for (const auto &gender : gender_type_map) {
-                const auto v = m_element.tagValue(gender.keyName);
-                if (v.isEmpty() || v == "no") {
-                    continue;
-                }
-                l.push_back(gender.label.toString());
-            }
-            return QLocale().createSeparatedList(l);
-        }
+            return Localization::genderSegregation(m_element);
         case Wikipedia: return wikipediaUrl(m_element.tagValue(m_langs, "wikipedia", "brand:wikipedia", "species:wikipedia"));
         case Address: return QVariant::fromValue(OSMAddress(m_element));
         case Phone: return QString::fromUtf8(m_element.tagValue("contact:phone", "phone", "telephone", "operator:phone"));
@@ -757,18 +905,19 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
         case Website: return QString::fromUtf8(m_element.tagValue("website", "contact:website", "url", "operator:website"));
         case PaymentCash:
         {
+            // TODO deal with the case that the tag value is a list of supported coins/notes (e.g. on vending machines)
             const auto coins = m_element.tagValue("payment:coins");
             const auto notes = m_element.tagValue("payment:notes");
             if (coins.isEmpty() && notes.isEmpty()) {
                 return translatedBoolValue(m_element.tagValue("payment:cash"));
             }
-            if (coins == "yes" && notes == "yes") {
+            if (!coins.isEmpty() && !notes.isEmpty() && coins != "no" && notes != "no") {
                 return i18n("yes");
             }
-            if (coins == "yes") {
+            if (!coins.isEmpty() && coins != "no" && (notes == "no" || notes.isEmpty())) {
                 return i18nc("payment option", "coins only");
             }
-            if (notes == "yes") {
+            if (!notes.isEmpty() && notes != "no" && (coins == "no" || coins.isEmpty())) {
                 return i18nc("payment option", "notes only");
             }
             return i18n("no");
@@ -842,12 +991,26 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
 
 QVariant OSMElementInformationModel::urlify(const QVariant& v, OSMElementInformationModel::Key key) const
 {
-    if (v.userType() != QMetaType::QString) {
+    if (v.userType() != QMetaType::QString && key != Image && key != Logo) {
         return v;
     }
     const auto s = v.toString();
 
     switch (key) {
+        case Image:
+        case Logo:
+        {
+            if (const auto commons = m_element.tagValue("wikimedia_commons"); commons.startsWith("File:")) {
+                return QUrl(u"https://commons.wikimedia.org/wiki/"_s + QString::fromUtf8(commons));
+            }
+            if (const QUrl url(QString::fromUtf8(m_element.tagValue("image"))); url.host() == "commons.wikimedia.org"_L1) {
+                return wikimediaCommondRedirect(url.fileName());
+            }
+            if (const auto wdId = m_element.tagValue("wikidata", "species:wikidata", "genus:wikidata", "subject:wikidata", "operator:wikidata", "network:wikidata", "brand:wikidata"); !wdId.isEmpty()) {
+                return QUrl(u"https://wikidata.org/wiki/" + QString::fromUtf8(wdId));
+            }
+            return {};
+        }
         case Email:
             if (!s.startsWith(QLatin1String("mailto:"))) {
                 return QString(QLatin1String("mailto:") + s);
